package SA::Network::Wireless;
use strict;use vars qw(@ISA $VERSION);
use Fcntl qw(:flock);
use SA;
use SA::MW;
use SA::LCD::Disp;

@ISA = qw(SA::MW);
$VERSION = '1.0';

=head1 NAME

SA::Network::Wireless - get and set wireless access point configuration

=head1 SYNOPSIS

use SA::Network::Wireless;

my $fw = SA::Network::Wireless->new();

$fw->

=head1 DESCRIPTION

Enable and "disable" wireless access point settings.  Will
provide a list of optional rulesets which will allow certain predefined
services access through the wireless access point.

=head1 METHODS

=over 4

=item new([CFG])

Returns an object instance against which the remaining methods may be called.
Provide the optional CFG (SA::Config object) or the constructor will get one of
its own from SA::MAIN_CONF.

=cut

sub new
{
	my $class = shift;
	my $cfg = shift;

	$class->SUPER::new($cfg, 'mod_wireless');
}

=item mod_attributes(HASH)

Set wireless network attributes.

=cut

sub mod_attributes
{
	my $self = shift;
	my %params = @_;

	eval
	{
#		if ($self->_check_all_params("", \%params))
#		{
			foreach my $param (keys %params)
			{
				$self->{_cfg}->set_leaf("network.wireless.$param", "$params{$param}")
						or die $self->{_cfg}->get_last_error()->error();
			}
#		}
	};

	return $self->_set_error($@) if $@;
	
	return $self->_unset_error();
}

=item mod_attributes(HASH)

Get wireless network attributes.

=cut

sub get_attributes
{
	my $self = shift;

	my $prop = $self->{_cfg}->get_tree("network.wireless") || {};

	return $prop;
}

=item get_wireless_installed()

Determines whether wireless is installed

=cut

sub get_wireless_installed
{
	my $installed = 0;
	my $is_prism_card = 0;

#trace_device("/sa2/Wirelesspm.txt");
#trace("write: file=$filename, val=$value\n");
	# read the loaded modules data from /proc
	open(P, "/proc/modules") or die;
	while (<P>) 
	{
		if (/hostap_cs/) 
# 		if (/mii/) 
		{
			$installed = 1;
			$is_prism_card = 1;
		}
 		if (/tsap wavelan/) 
		{
			$installed = 1;
		}
	}

#	trace("Check 1: installed=$installed is_prism_card=$is_prism_card\n");
	close(P);

	if ($installed)
	{
		my $ap_dev;
		if ($is_prism_card) {
			if (open(DEV, "/etc/sysconfig/tsb/tsap_dev"))
			{
				while (<DEV>)
				{
					chomp;
					if ($_ ne "")
					{
						$ap_dev = $_;
						last;
					}
				}
				close(DEV);
			}

			open WIRELESS, "/proc/net/hostap/".$ap_dev or $installed=0;
#			open WIRELESS, "/proc/net/atm" or $installed=0;
			close WIRELESS;
		}
		else {
                	$installed = 0;
                        open WIRELESS, "/proc/net/wireless";

                        while (<WIRELESS>)
                        {
                                if (/eth2:/)
                                {
                                        $installed = 1;
                                        last;
                                }
                        }
                        close WIRELESS;
		}
#		trace("Check 2: installed=$installed is_prism_card=$is_prism_card\n");
 	
	}
	return $installed;
}

=item get_enabled(), enable_wireless(), disable_wireless()

Gets and sets enabled state of wireless access point.

=cut

sub get_enabled
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.enabled');
}

sub enable_wireless
{
	my $self = shift;
	$self->{_cfg}->set_leaf('network.wireless.enabled', 1);
	return 1;
}

sub disable_wireless
{
	my $self = shift;
	$self->{_cfg}->set_leaf('network.wireless.enabled', 0);
	return 1;
}


sub is_eth2_linked
{
#trace_device("/sa2/Wirelesspm.txt");
	my $retval = 0;
	open WIRELESS, "/proc/net/wireless";

	while (<WIRELESS>)
	{
		if (/eth2:/)
		{
			my ($device, $status, $link, $level) = split ' ', $_, 4; 
			if ($link ne '0' && $level ne '0')
			{
				$retval = 1;
				last;
			}
		}
	}
	
	close WIRELESS;

	return $retval;
}

sub is_eth2_up
{
	my $retval = 0;
	open IFCONFIG, "/proc/net/dev";

	while (<IFCONFIG>)
	{
		if (/^\s+eth2:/)
		{
			$retval = 1;
			last;
		}
	}
	
	close IFCONFIG;

	return $retval;
}

sub _check_enabled
{
	my $self = shift;
	my $if = shift;
	my $params = shift;

	die 'network.wireless.invalid_enabled_flag' unless
			$params->{enabled} == 1 || $params->{enabled} == 0;

	return $self->_unset_error();
}

=item is_wireless_on(), turn_wireless_on(), turn_wireless_off()

Brings up connection for wireless access point.

=cut

sub is_wireless_on
{
	my $self = shift;
	return is_eth2_linked();
}

sub turn_wireless_on
{
	my $self = shift;

	my $retval = 0;

	if (get_wireless_installed() && (!is_eth2_linked()) &&
		system("/sbin/ifup","eth2") == 0)
 	{
 		$retval = 1;
 	}

	return $retval;
}

sub turn_wireless_off
{
	my $self = shift;
	my $retval = 0;

	if (is_eth2_linked() && system("/sbin/ifdown","eth2") == 0)
	{
		$retval = 1;
	}

	return $retval;
}


=item get_network_name(), set_network_name([NAME])

Gets and sets network name of wireless access point.

=cut

sub get_network_name
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.network_name');
}

sub set_network_name
{
	my $self = shift;
	my $name = shift;
	return $self->{_cfg}->set_leaf('network.wireless.network_name', $name);
}

sub _check_network_name
{
	my $self = shift;
	my $if = shift;
	my $params = shift;

#	die 'network.wireless.invalid_network_name' unless
#			$params->{network_name} =~ /
#			[^\d]?    # optional non-digit
#			(\d{3})?  # optional areacode
#			[^\d]?    # optional non-digit
#			\d{3}     # three-digit prefix
#			[^\d]?    # optional non-digit
#			\d{4}     # four-digit number
#			/x;

	return $self->_unset_error();
}

=item get_station_name(), set_station_name([NAME])

Gets and sets station name of wireless access point.

=cut

sub get_station_name
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.station_name');
}

sub set_station_name
{
	my $self = shift;
	my $name = shift;
	return $self->{_cfg}->set_leaf('network.wireless.station_name', $name);
}

=item get_reject_any(), set_reject_any([FLAG])

Gets and sets flag determining whether access point will accept any network name.

=cut

sub get_reject_any
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.reject_any');
}

sub set_reject_any
{
	my $self = shift;
	my $flag = shift;
	return $self->{_cfg}->set_leaf('network.wireless.reject_any', $flag);
}

=item get_channel(), set_channel([CHANNEL])

Gets and sets channel.

=cut

sub get_channel
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.channel');
}

sub set_channel
{
	my $self = shift;
	my $channel = shift;
	return $self->{_cfg}->set_leaf('network.wireless.channel', $channel);
}

=item get_enable_mac(), set_enable_mac([FLAG])

Gets and sets flag determining whether access point will filter mac addresses.

=cut

sub get_enable_mac
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.enable_macfilter');
}

sub set_enable_mac
{
	my $self = shift;
	my $flag = shift;
	return $self->{_cfg}->set_leaf('network.wireless.enable_macfilter', $flag);
}

=item get_enable_encryption(), set_enable_encryption([FLAG])

Gets and sets flag determining whether access point will use wep.

=cut

sub get_enable_encryption
{
	my $self = shift;
trace_device("/sa2/Wirelesspm.txt");
trace("get_enable_encryption\n");
	return $self->{_cfg}->get_leaf('network.wireless.enable_encryption');
}

sub set_enable_encryption
{
	my $self = shift;
	my $flag = shift;	
trace_device("/sa2/Wirelesspm.txt");
trace("set_enable_encryption\n");
	my $is_prism_card=0; open(P, "/proc/modules") or die; while (<P>) { if (/hostap_cs/) { $is_prism_card = 1; } } close(P);
	if ($is_prism_card) {
		trace("set_enable_encryption prism_card $flag\n");
		if ($flag) { 
			system("/usr/bin/logger","-i","iwconfig eth2 key restricted");
			system("/sbin/iwconfig","eth2","key","restricted");
		} 
		else { 
			system("/usr/bin/logger","-i","iwconfig eth2 key open");
			system("/sbin/iwconfig","eth2","key","open");
		}
	}
	else {
		write_wep_item("/proc/sys/net/wireless/enable_encryption", $flag);
	}

#	save_wep_info($self);

	return $self->{_cfg}->set_leaf('network.wireless.enable_encryption', $flag);
}

sub save_wep_info{
	my $self = shift;
	my $success = 0;
	my $keystr = "";
trace_device("/sa2/Wirelesspm.txt");
	my $is_prism_card=0; open(P, "/proc/modules") or die; while (<P>) { if (/hostap_cs/) { $is_prism_card = 1; } } close(P);

	if ($is_prism_card) {
		$keystr=`/sbin/iwconfig eth2|grep Encryption`;
		$keystr=substr($keystr,25,32);
		trace("save_wep_info: keystr=$keystr\n");
	}
	else {
		if (open(PROC, "/proc/sys/net/wireless/keys"))
		{
			while (<PROC>)
			{
				chomp;
				if ($_ ne "")
				{
					$keystr = $_;
#			trace("save_wep_info: keystr=$keystr\n");
					last;
				}
			}
			close(PROC);
		}
	}

	if ($keystr ne "")
	{
		$success = $self->{_cfg}->set_leaf('network.wireless.keys', $keystr);
	}

	return $success;
}

sub write_wep_item
{
	my $filename = shift;
	my $value = shift;
	my $success = 0;
	
#trace_device("/sa2/Wirelesspm.txt");
#trace("write: file=$filename, val=$value\n");
	if (open(PROC, ">$filename"))
	{
		flock(PROC, LOCK_EX);
		print PROC $value."\n";
		flock(PROC, LOCK_UN);
		close(PROC);
		$success = 1;

	}
		
	return $success;
}

=item get_keys(), set_keys([KEYS])

Gets and sets encryption keys.  Keys must be passed as hashref

=cut

sub get_keys
{
	my $self = shift;
#trace_device("/sa2/Wirelesspm.txt");
#trace("get_keys\n");
	return $self->{_cfg}->get_tree('network.wireless.keylist');
}

sub set_keys
{
trace_device("/sa2/Wirelesspm.txt");
trace("set_keys\n");
	my $self = shift;
	my $enckeys = shift;
	my %enckeys = %$enckeys;
	my $hexlen = get_128_support() ? 26 : 10;
	my $counter = 0;
	
	my $is_prism_card=0; open(P, "/proc/modules") or die; while (<P>) { if (/hostap_cs/) { $is_prism_card = 1; } } close(P);

	if ($is_prism_card) {
		foreach my $k (keys %enckeys)
		{
			$counter++;
#			trace("Key $counter\n");
			if ($counter > 1) {
				$enckeys->{$k} = "";
#				trace("Prism-Card, Key $counter deleted\n");
				next;
			}
#			trace("Key: $enckeys->{$k}\n");
			if ((length($enckeys{$k}) == $hexlen) && ($enckeys->{$k} ne "*****"))
			{
				$enckeys->{$k} = 
					substr($enckeys->{$k},0,4)."-".
					substr($enckeys->{$k},4,4)."-".
					substr($enckeys->{$k},8,4)."-".
					substr($enckeys->{$k},12,4)."-".
					substr($enckeys->{$k},16,4)."-".
					substr($enckeys->{$k},20,4)."-".
					substr($enckeys->{$k},24,2);
			}
			else {
				$enckeys->{$k} = "s:".$enckeys->{$k};
			}
#			trace("Newkey: $enckeys->{$k}\n");
			
			if ($enckeys->{$k} ne "*****")		
			{
#				trace("Setting Key now: $enckeys->{$k}\n");
				system("/usr/bin/logger","-i","iwconfig eth2 key",$enckeys->{$k});
				system("/sbin/iwconfig","eth2","key",$enckeys->{$k});
			}

			if ($enckeys->{$k} ne "")		
			{
				$enckeys->{$k} = '*****';
			}	
		}	
	} 
	else {
	foreach my $k (keys %enckeys)	
	{
		if (length($enckeys{$k}) == $hexlen)
		{
			$enckeys->{$k} = "0x".$enckeys->{$k};
		}
		
		if ($enckeys->{$k} ne "*****")		
		{
			write_wep_item(
				"/proc/sys/net/wireless/key".$k, 
				$enckeys->{$k});
		}

		if ($enckeys->{$k} ne "")		
		{
			$enckeys->{$k} = '*****';
		}	
	}
	}	
#sleep(10);
#trace_device("/sa2/Wirelesspm.txt");
#trace("set_keys (sleeping well)\n");
#	save_wep_info($self);
	if ($self->{_cfg}->set_leaf('network.wireless.keylist', ''))
	{
		return $self->{_cfg}->set_tree('network.wireless.keylist', $enckeys);
	}
	return 0;
}

=item get_transmit_key_id(), set_transmit_key_id([KEY])

Gets and sets selected encryption key index.

=cut

sub get_transmit_key_id
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.transmit_key_id');
}

sub set_transmit_key_id
{
	my $self = shift;
	my $key = shift;
#trace_device("/sa2/Wirelesspm.txt");
#trace("set_transmit_key_id\n");
	my $is_prism_card=0; open(P, "/proc/modules") or die; while (<P>) { if (/hostap_cs/) { $is_prism_card = 1; } } close(P);
	if ($is_prism_card) {
		$key = 1;
#		trace("Prism-Card, ID always 1\n");
	}
	else {
		write_wep_item("/proc/sys/net/wireless/transmit_key_id",$key);
	}
#	save_wep_info($self);
	return $self->{_cfg}->set_leaf('network.wireless.transmit_key_id', $key);
}

=item get_key_input_mode(), set_key_input_mode([MODE])

Gets and sets mode used to enter encryption keys.

=cut

sub get_key_input_mode
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.keyinputmode');
}

sub set_key_input_mode
{
	my $self = shift;
	my $keys = shift;
	return $self->{_cfg}->set_leaf('network.wireless.keyinputmode', $keys);
}

=item get_mac_accept(), set_mac_accept([VALUES])

Gets and sets accepted mac values.

=cut

sub get_mac_accept
{
	my $self = shift;
	return $self->{_cfg}->get_tree('network.wireless.mac_accept');
}

sub set_mac_accept
{
	my $self = shift;
	my $values = shift;
	if ($self->{_cfg}->set_leaf('network.wireless.mac_accept', ''))
	{
		return $self->{_cfg}->set_tree('network.wireless.mac_accept', $values);
	}
	return 0;
}

=item get_multicast_rate(), set_multicast_rate([RATE])

Gets and sets multicast rate.

=cut

sub get_multicast_rate
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.multicast_rate');
}

sub set_multicast_rate
{
	my $self = shift;
	my $rate = shift;
	return $self->{_cfg}->set_leaf('network.wireless.multicast_rate', $rate);
}

=item get_distance_between_aps(), set_distance_between_aps([DISTANCE])

Gets and sets distance between access points.

=cut

sub get_distance_between_aps
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.distance_between_aps');
}

sub set_distance_between_aps
{
	my $self = shift;
	my $distance = shift;
	return $self->{_cfg}->set_leaf('network.wireless.distance_between_aps', $distance);
}

=item get_dtim_period(), set_dtim_period([PERIOD])

Gets and sets dtim period.

=cut

sub get_dtim_period
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.wireless.dtim_period');
}

sub set_dtim_period
{
	my $self = shift;
	my $period = shift;
	return $self->{_cfg}->set_leaf('network.wireless.dtim_period', $period);
}

=item get_128_support()

Does card in server appliance support 128 bit WEP.

=cut

sub get_128_support
{
#trace_device("/sa2/Wirelesspm.txt");
	my $installed = 0;
	my $is_prism_card = 0;

	# read the loaded modules data from /proc
	open(P, "/proc/modules") or die;
	while (<P>) 
	{
 		if (/hostap_cs/) 
# 		if (/mii/) 
		{
			$installed = 1;
			$is_prism_card = 1;
		}
                if (/tsap wavelan/)
                {
                        $installed = 1;
                }
	}
	close(P);
#	trace("128bitCheck: installed=$installed is_prism_card=$is_prism_card\n");

        if (($installed) && ($is_prism_card == 0))
        {
                $installed = 0;
                open CARDPROFILE, "/proc/sys/net/wireless/card_profile" or return 0;

                while (<CARDPROFILE>)
                {
                        chomp;                  if ($_ ne "")
                        {
                                my ($profile,$capability,$rates) = split(" ", $_);

                                if ($capability & 2)
                                {
                                        $installed = 1;
                                }
                                last;
                        }               }
                close CARDPROFILE;

        }
        if (($installed) && ($is_prism_card))
	{
		# any prism2 card supports 128 bit encryption 
		# so just do nothing here
	}
	
	return $installed;
}

=item get_status()

Get the wireless networking status

=cut

sub get_status
{
	my $self = shift;
	my $lcdResult = "wireless.notinstalled";

	if (get_wireless_installed())
	{
		if (is_eth2_linked())
		{
			if (get_128_support())
			{
				$lcdResult = "wireless.128enabled";
			}
			else
			{
				$lcdResult = "wireless.64enabled";
			}
		}
		else
		{
			$lcdResult = "wireless.disabled";
		}
	}
	else
	{
		$lcdResult = "wireless.notinstalled";
	}
	return $lcdResult;
}

=item display_lcd_status()

Displays wireless status on lcd

=cut

sub display_lcd_status
{
#trace_device("/sa2/Wirelesspm.txt");

	my $lcdDisp = new SA::LCD::Disp;

	return $lcdDisp->ShowMsg(get_status());
}

1;

__END__

=head1 AUTHOR

Cass Cunningham <cass.cunningham@tais.com>

=head1 SEE ALSO

perl(1), SA::Config(3).

=cut
